@@ -35,10 +35,12 @@
    D_RGB_PERSIST,
    D_RGB_BITDIST,
    D_RGB_OPERM,
+   D_RGB_PERMUTATIONS,
    D_RGB_LMN,
    D_RGB_MINIMUM_DISTANCE,
    D_STS_MONOBIT,
    D_STS_RUNS,
+   D_STS_SERIAL,
    D_STS_BLOCK,
    D_USER_TEMPLATE,
    D_MEASURE,
@@ -48,10 +50,11 @@
    D_SAMPLE,
    D_CHISQ,
    D_KSTEST,
-   D_BTEST,
+   D_VTEST,
    D_XTEST,
    D_BRANK,
    D_FILE_INPUT,
    D_FILE_INPUT_RAW,
+   D_ADD_MY_TYPES,
    N_DEBUG
  } Debug;
